﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace MoonLanding
{
    public partial class GameWindow : Form
    {
        string address;
        LandingMode mode;

        uint id;
        PointF position, velocity;
        float fuel;
        float landingPosL, landingPosR;
        int thrusterL, thrusterR, thrusterU;

        Image landerImg;
        float scale = 1f;
        float offsetX = 0f;

        public GameWindow(string host, LandingMode _mode)
        {
            InitializeComponent();
            landerImg = Properties.Resources.lunar_lander;
            address = host;
            mode = _mode;
            this.Paint += GameWindow_Paint;
            UpdateFromServer(true);
            this.updateLoop.Enabled = true;
        }

        private void GameWindow_KeyDown(object sender, KeyEventArgs e)
        {
            if (mode == LandingMode.Automatic) return;
            switch (e.KeyCode)
            {
                case Keys.Up:
                    thrusterU = 1;
                    break;
                case Keys.Left:
                    thrusterL = 1;
                    break;
                case Keys.Right:
                    thrusterR = 1;
                    break;
            }
            UpdateFromServer(false, "&nahoru=" + thrusterU + "&doleva=" + thrusterL + "&doprava=" + thrusterR);
        }

        private void GameWindow_KeyUp(object sender, KeyEventArgs e)
        {
            if (mode == LandingMode.Automatic) return;
            switch (e.KeyCode)
            {
                case Keys.Up:
                    thrusterU = 0;
                    break;
                case Keys.Left:
                    thrusterL = 0;
                    break;
                case Keys.Right:
                    thrusterR = 0;
                    break;
            }
            UpdateFromServer(false, "&nahoru=" + thrusterU + "&doleva=" + thrusterL + "&doprava=" + thrusterR);
        }

        private void GameWindow_Paint(object sender, PaintEventArgs e)
        {
            //Calculate dynamic zoom scale and X offset
            /*if (dynamicZoom.Checked)
            {
                float maxDist = Math.Max(Math.Abs(position.X - ((landingPosL + landingPosR)/2)), position.Y - 20f);
                scale = 1 / 200 * maxDist;
                //offsetX = 
            }
            else
            {
                offsetX = 0f;
                scale = 1f;
            }*/

            //Landing area
            e.Graphics.DrawLine(Pens.Black, offsetX + landingPosL * scale, 200 * scale, landingPosR * scale, 200 * scale);
            //Lander
            e.Graphics.DrawImage(landerImg , (position.X - 10) * scale + offsetX, (190 - position.Y) * scale, 20 * scale, 20 * scale);
            //Fuel ( range 0 ... 20 )
            e.Graphics.DrawRectangle(Pens.Black, 5, 205, 70, 20);
            e.Graphics.FillRectangle(Brushes.Red, 6, 206, fuel / 20f * 69f, 19);
            //Velocity
            Font font = new Font(FontFamily.GenericSansSerif, 15f);
            e.Graphics.DrawString("Velocity X: " + velocity.X, font,Brushes.Black, 5, 230);
            e.Graphics.DrawString("Velocity Y: " + velocity.Y, font, Brushes.Black, 5, 250);
            //Thrusters
            if (thrusterL == 1)
            {
                e.Graphics.DrawImage(Properties.Resources.arrow_left, 110, 205, 30, 30);
            }
            if (thrusterU == 1)
            {
                e.Graphics.DrawImage(Properties.Resources.arrow_up, 150, 205, 30, 30);
            }
            if (thrusterR == 1)
            {
                e.Graphics.DrawImage(Properties.Resources.arrow_right, 190, 205, 30, 30);
            }
        }

        //TODO: Implement local updates later...
        private void UpdateLocally(int timePassed)
        {
            float timeInSeconds = timePassed / 1000f;
            position.X = position.X + (velocity.X * timePassed) + ((1 / 2) * (6 * thrusterR - (6 * thrusterL)) * timePassed * timePassed);
            //position.Y = pos
        }

        private void UpdateFromServer(bool start, string thrusters = "")
        {
            if (mode == LandingMode.Automatic && !start)
            {
                //Vertical landing (works... sometimes...)
                var test = (velocity.Y * velocity.Y) / 7.5f; //7.5
                if (position.Y - 10 < test)
                {
                    thrusterU = 1;
                }
                if (Math.Abs(velocity.Y) < 2)
                    thrusterU = 0;
                
                //Horizontal 
                thrusterL = 0;
                thrusterR = 0;
                
                if (position.X - 10 < landingPosL && (Math.Abs(velocity.X) < 15))
                    thrusterR = 1;
                else if (position.X + 10 > landingPosR && (Math.Abs(velocity.X) < 15))
                    thrusterL = 1;
                else if (velocity.X > 4)
                    thrusterL = 1;
                else if (velocity.X < -4)
                    thrusterR = 1;

                //Override any other thruster input
                thrusters = "&nahoru=" + thrusterU + "&doleva=" + thrusterL + "&doprava=" + thrusterR; 
            }

            //Prepare and send the request
            string command = start ? "start" : "stav?id=" + id;
            var request = HttpWebRequest.CreateHttp(address + "/" + command + thrusters);
            WebResponse response;
            try
            {
                response = request.GetResponse();
            }
            catch (Exception ex)
            {
                MessageBox.Show("Failed to get response for given URL. " + ex.Message, "Exception caught", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            var stream = response.GetResponseStream();
            StreamReader reader = new StreamReader(stream);

            //Line 1 - ID / landing result
            string line1 = reader.ReadLine(); ;
            if (!uint.TryParse(line1, out id))
            {
                updateLoop.Stop();
                MessageBox.Show("Výsledek přistání: " + line1);
                this.Close();
                return;
            }
            //  CultureInfo.InvariantCulture is needed to parse float with "." as decimal separator
            //Line 2 - postion
            string[] XY = reader.ReadLine().Split(' ');
            position.X = float.Parse(XY[0], CultureInfo.InvariantCulture);
            position.Y = float.Parse(XY[1], CultureInfo.InvariantCulture);
            //Line 3 - velocity
            string[] vXY = reader.ReadLine().Split(' ');
            velocity.X = float.Parse(vXY[0], CultureInfo.InvariantCulture);
            velocity.Y = float.Parse(vXY[1], CultureInfo.InvariantCulture);
            //Line 4 - fuel
            fuel = float.Parse(reader.ReadLine(), CultureInfo.InvariantCulture);
            //Line 5 - landing location
            string[] landPos = reader.ReadLine().Split(' ');
            landingPosL = float.Parse(landPos[0], CultureInfo.InvariantCulture);
            landingPosR = float.Parse(landPos[1], CultureInfo.InvariantCulture);
        }

        private void updateLoop_Tick(object sender, EventArgs e)
        {
            UpdateFromServer(false);
            this.Invalidate();
        }
    }
}
